@@ -22,7 +22,6 @@
 #include <config.h>
 
 #include "rhythmdb.h"
-#include "rhythmdb-model.h"
 #include "rhythmdb-legacy.h"
 #include "rhythmdb-query-model.h"
 #include "rhythmdb-property-model.h"
@@ -539,6 +538,7 @@ typedef struct
 	GValue file_size_val;
 	GValue title_val;
 	GValue duration_val;
+	GValue quality_val;
 	GValue mtime_val;
 	GValue genre_val;
 	GValue artist_val;
@@ -553,6 +553,7 @@ read_metadata (const char *location, GError **error)
 	RhythmDBEntryUpdateData *data = g_new0 (RhythmDBEntryUpdateData, 1);
 	MonkeyMediaStreamInfo *info;
 	GnomeVFSFileInfo *vfsinfo;
+	GValue tem = {0,};
 
 	info = monkey_media_stream_info_new (location, error);
 	if (G_UNLIKELY (info == NULL)) {
@@ -573,6 +574,13 @@ read_metadata (const char *location, GError **error)
 				            MONKEY_MEDIA_STREAM_INFO_FIELD_DURATION,
 					    0, &data->duration_val);
 
+	/* quality */
+	monkey_media_stream_info_get_value (info,
+				            MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_QUALITY,
+					    0, &tem);
+	g_value_init (&data->quality_val, G_TYPE_INT);
+	g_value_set_int (&data->quality_val, g_value_get_enum (&tem));
+
 	/* filesize */
 	monkey_media_stream_info_get_value (info, MONKEY_MEDIA_STREAM_INFO_FIELD_FILE_SIZE,
 					    0, &data->file_size_val);
@@ -629,6 +637,8 @@ synchronize_entry_with_data (RhythmDB *db, RhythmDBEntry *entry, RhythmDBEntryUp
 	g_value_unset (&data->track_number_val);
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &data->duration_val);
 	g_value_unset (&data->duration_val);
+	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_QUALITY, &data->quality_val);
+	g_value_unset (&data->quality_val);
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_FILE_SIZE, &data->file_size_val);
 	g_value_unset (&data->file_size_val);
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_TITLE, &data->title_val);
@@ -1317,6 +1327,7 @@ write_encoded_gvalue (xmlNodePtr node,
 		break;
 	default:
 		g_assert_not_reached ();
+		strval = NULL;
 		break;
 	}
 
@@ -1494,8 +1505,9 @@ query_thread_main (struct RhythmDBQueryThreadData *data,
 
 	rb_debug ("doing query");
 
-	if (data->lock)
-		rhythmdb_read_lock (db);
+	/* Async queries now expect the db to be locked */
+/* 	if (data->lock) */
+/* 		rhythmdb_read_lock (db); */
 	
 	klass->impl_do_full_query (db, data->query,
 				   data->main_model,
@@ -1520,6 +1532,8 @@ rhythmdb_do_full_query_async_parsed (RhythmDB *db, GtkTreeModel *main_model,
 {
 	struct RhythmDBQueryThreadData *data;
 
+	db_enter (db, FALSE);
+
 	data = g_new0 (struct RhythmDBQueryThreadData, 1);
 	data->query = rhythmdb_query_copy (query);
 	data->main_model = main_model;
@@ -1569,7 +1583,7 @@ rhythmdb_do_full_query_internal (RhythmDB *db, GtkTreeModel *main_model,
 	g_object_ref (G_OBJECT (main_model));
 	db->priv->outstanding_threads++;
 	query_thread_main (data, db);
-	while (rhythmdb_model_poll (RHYTHMDB_MODEL (main_model), NULL))
+	while (rhythmdb_query_model_poll (RHYTHMDB_QUERY_MODEL (main_model), NULL))
 		;
 }
 
